function gamma_i_ast = calcGamma_InverseGamma(A, B, b, rho, threshold, gamma_old)

if nargin < 5
  threshold = 1;
end

map_numerator = b*A;
map_denominator = 2*rho*B;



poly_coeff = map_numerator - [map_denominator; 0];


direction_increasing = polyval(poly_coeff, gamma_old) > 0;

% calculate stationary points as roots of the polynomial
if any(isnan(poly_coeff)) || any(isinf(poly_coeff))
  gamma_i_ast = inf;
  return
end

gamma_i_ast = roots(poly_coeff);
% Sanity- check by using fzero to also search for a real root.
% gamma_i_ast = [gamma_i_ast; fzero(@(x) polyval(poly_coeff, x), 0)];

% only acceptable stationary points that are positve and real
gamma_i_ast = real(gamma_i_ast(~isnan(gamma_i_ast) & imag(gamma_i_ast)==0 & real(gamma_i_ast)>0));

if isempty(gamma_i_ast)
  gamma_i_ast = inf;
  return
end

% calculate which of the roots are attractors
% U_prime = polyder(U);
% V_prime = polyder(V);
% UpV_UVp = conv(U_prime, V) - conv(U, V_prime);
% V_squared = conv(V, V);
if threshold < 1
    [derivative_numerator, derivative_denominator] = polyder(map_numerator, map_denominator);
    dF = 1/2 * sqrt(polyval(map_denominator, gamma_i_ast))./sqrt(polyval(map_numerator, gamma_i_ast)) ...
      * polyval(derivative_numerator, gamma_i_ast)./polyval(derivative_denominator, gamma_i_ast);
    
    % check if any positve stationary point is an attractor
    gamma_i_ast = gamma_i_ast(abs(dF)<threshold);
    dF = dF(abs(dF)<threshold);
end

if direction_increasing
    if all(gamma_i_ast < gamma_old)
        gamma_i_ast = inf;
    else
        gamma_i_ast = min(gamma_i_ast(gamma_i_ast > gamma_old));
    end
else % if direction is decreasing
    gamma_i_ast = max(gamma_i_ast(gamma_i_ast<gamma_old));
    if isempty(gamma_i_ast)
        warning('Did not find a smaller stationary point although it should exist')
        gamma_i_ast = 0;
    end
end
end



