function [A, B] = getPolynomialsAB(q_k_squared, s_k)
% function GETPOLYNOMIALSAB calculates coefficients of polynomials A and B
%   [A, B] = getPolynomialsAB(q_k_squared, s_k) calculates the coeffients of the
%   polynomials A_i and B_i such that the expectation <x_i B_i^(-1) x_i>
%   over the distribution q_x equals B_i(alpha_i)/A_i(alpha_i)


L = numel(s_k);

D = zeros(3, L);
C = zeros(2, L);

D(1, :) = s_k.^2;
D(2, :) = 2 * s_k;
D(3, :) = 1;

C(1, :) = s_k.^2;
C(2, :) = (q_k_squared + s_k);

%% Orignal way to calculate A and B (directly as in the paper)
% A = L*polyprod(D);
% B = conv(C(:, 1), polyprod(D(:, 2:end)));
% for ll=2:L
%   idx_lbar = [1:ll-1, ll+1:L];
%   B = B+conv(C(:, ll), polyprod(D(:, idx_lbar)));
% end

%% Alternative, faster way to calculate A and B
if L==1
  A = D;
  B = C;
else
  F{1} = D(:, 1);
  G{1} = D(:, end);
  
  for ll=2:L-1
    F{ll} = conv(F{ll-1}, D(:, ll));
    G{ll} = conv(G{ll-1}, D(:, end-ll+1));
  end
  
  A = conv(F{end}, D(:, end));
  B = conv(G{end}, C(:, 1));
  for ll=2:L-1
    B = B+conv(conv(C(:, ll), F{ll-1}), G{end-ll+1});
  end
  B = B + conv(F{end}, C(:, end));
end



end